package net.parsell.cherry.core;

import net.fabricmc.fabric.api.object.builder.v1.block.FabricBlockSettings;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.FenceBlock;
import net.minecraft.block.FenceGateBlock;
import net.minecraft.block.LeavesBlock;
import net.minecraft.block.PillarBlock;
import net.minecraft.block.PressurePlateBlock;
import net.minecraft.block.SignBlock;
import net.minecraft.block.SlabBlock;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.block.entity.SignBlockEntity;
import net.minecraft.util.SignType;
import net.parsell.cherry.common.blocks.CherryButtonBlock;
import net.parsell.cherry.common.blocks.CherryDoorBlock;
import net.parsell.cherry.common.blocks.CherryPressurePlate;
import net.parsell.cherry.common.blocks.CherryStairsBlock;
import net.parsell.cherry.common.blocks.CherryTrapdoorBlock;

public class CherryBlocks {
	public static final Block CHERRYLOGSTRIPPED = createPillar("stripped_cherry_log", FabricBlockSettings.copyOf(Blocks.STRIPPED_OAK_LOG), 5, 5);
	public static final Block CHERRYLOG = createPillar("cherry_log", FabricBlockSettings.copyOf(Blocks.OAK_LOG), 5, 5, CHERRYLOGSTRIPPED);
	public static final Block CHERRYWOOD = createPillar("cherry_wood", FabricBlockSettings.copyOf(Blocks.OAK_WOOD), 5, 5);
	public static final Block CHERRYWOODSTRIPPED = createPillar("stripped_cherry_wood", FabricBlockSettings.copyOf(Blocks.STRIPPED_OAK_WOOD), 5, 5);
	public static final Block CHERRYLEAVES = createLeaves("cherry_leaves", FabricBlockSettings.copyOf(Blocks.OAK_LEAVES), 30, 60, 6649929);
	public static final Block CHERRYPLANKS = createBlock("cherry_planks", FabricBlockSettings.copyOf(Blocks.OAK_PLANKS), 5, 20);
	public static final Block CHERRYFENCE = createFence("cherry_fence", FabricBlockSettings.copyOf(Blocks.OAK_FENCE), 5, 20); // TODO: tags (as required)
	public static final Block CHERRYFENCEGATE = createFenceGate("cherry_fence_gate", FabricBlockSettings.copyOf(Blocks.OAK_FENCE_GATE), 5, 20); // TODO: recipe, loot_tables, tags (as required)
	public static final Block CHERRYDOOR = createDoor("cherry_door", FabricBlockSettings.copyOf(Blocks.OAK_DOOR), 5, 20); // TODO: recipe, loot_tables, tags (as required)
	public static final Block CHERRYTRAPDOOR = createTrapDoor("cherry_trapdoor", FabricBlockSettings.copyOf(Blocks.OAK_TRAPDOOR), 5, 20); //TODO: recipe, loot_tables, tags (as required)
	public static final Block CHERRYSLAB = createSlab("cherry_slab", FabricBlockSettings.copyOf(Blocks.OAK_SLAB), 5, 20); // TODO: recipe, loot tables, tags (as required)
	public static final Block CHERRYSTAIRS = createStair("cherry_stairs", CHERRYPLANKS.getDefaultState(), FabricBlockSettings.copyOf(Blocks.OAK_STAIRS), 5, 20); //TODO: recipe, loot tables, tags (as required)
	public static final Block CHERRYBUTTON = createButton("cherry_button", FabricBlockSettings.copyOf(Blocks.OAK_BUTTON), 5, 20); //TODO: recipe, loot tables, tags (as required)
	public static final Block CHERRYPRESSUREPLATE = createPressurePlate("cherry_pressure_plate", PressurePlateBlock.ActivationRule.EVERYTHING, FabricBlockSettings.copyOf(Blocks.OAK_PRESSURE_PLATE), 5, 20); //TODO: recipe, loot tables, tags (as required)
	public static final SignType CHERRYSIGN_TYPE = CherryRegistry.register("cherry");
	public static final Block CHERRYSIGN = createSign("cherry_sign", CHERRYSIGN_TYPE, FabricBlockSettings.copyOf(Blocks.OAK_SIGN), 5, 20); // TODO: recipe, loot tables
	public static final BlockEntityType<SignBlockEntity> CHERRYSIGN_ENTITY = BlockEntityType.Builder.create(SignBlockEntity::new, CHERRYSIGN).build(null);

	private static Block createBlock(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new Block(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createSlab(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new SlabBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createStair(String blockID, BlockState baseBlockState, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new CherryStairsBlock(baseBlockState, settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createButton(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new CherryButtonBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createPressurePlate(String blockID, PressurePlateBlock.ActivationRule rule, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new CherryPressurePlate(rule, settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createSign(String blockID, SignType type, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new SignBlock(settings, type);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createFence(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new FenceBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createFenceGate(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread) {
		Block block = new FenceGateBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createPillar(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new PillarBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		return block;
	}

	private static Block createPillar(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread, Block strippedVariant){
		Block block = new PillarBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		CherryRegistry.addStrippable(block, strippedVariant);
		return block;
	}

	private static Block createLeaves(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread, int color){
		Block block = new LeavesBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		CherryRegistry.registerColor(block, color);
		return block;
	}

	private static Block createDoor(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new CherryDoorBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		CherryRegistry.registerTransparentLayer(block);
		return block;
	}

	private static Block createTrapDoor(String blockID, FabricBlockSettings settings, int burnChance, int burnSpread){
		Block block = new CherryTrapdoorBlock(settings);
		CherryRegistry.register(block, blockID);
		CherryRegistry.registerBurnable(block, burnChance, burnSpread);
		CherryRegistry.registerTransparentLayer(block);
		return block;
	}

	public static void init(){
		CherryRegistry.register(CHERRYSIGN_ENTITY, "cherry_sign");
	}
}