package net.parsell.cherry.core;

import net.parsell.cherry.Cherry;
import net.parsell.cherry.common.util.CherrySignType;
import net.parsell.cherry.mixin.*;

import com.google.common.collect.ImmutableMap.Builder;

import net.fabricmc.fabric.api.blockrenderlayer.v1.BlockRenderLayerMap;
import net.fabricmc.fabric.api.client.itemgroup.FabricItemGroupBuilder;
import net.fabricmc.fabric.api.client.rendering.v1.ColorProviderRegistry;
import net.fabricmc.fabric.api.registry.FlammableBlockRegistry;
import net.minecraft.block.Block;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.client.render.RenderLayer;
import net.minecraft.item.BlockItem;
import net.minecraft.item.ItemGroup;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Identifier;
import net.minecraft.util.SignType;
import net.minecraft.util.registry.Registry;

public class CherryRegistry {
    private static Builder<Block, Block> STRIPPABLE_BLOCKS = new Builder<Block, Block>();
    public static final ItemGroup CHERRY_GROUP = FabricItemGroupBuilder
    .create(new Identifier(Cherry.MODID, "general"))
    .icon(() -> new ItemStack(CherryItems.CherryPlanks))
    .build();

    public static void init(){
        AxeItemAccessor.getStrippedLogs().forEach((log, strippedLog) -> {
            STRIPPABLE_BLOCKS.put(log, strippedLog);
        });
        System.out.println("Cherry Registry initilizating...");
    }

    public static void finish(){
        AxeItemAccessor.setStrippedLogs(STRIPPABLE_BLOCKS.build());
		System.out.println("Cherry Registry setup correctly!");
    }

    public static void register(Block block, String id){
        Registry.register(Registry.BLOCK, new Identifier(Cherry.MODID, id), block);
    }
    
    public static void register(BlockItem item, String id){
        Registry.register(Registry.ITEM, new Identifier(Cherry.MODID, id), item);
    }

    public static void register(BlockEntityType<?> blockEntityType, String id){
        Registry.register(Registry.BLOCK_ENTITY_TYPE, new Identifier(Cherry.MODID, id), blockEntityType);
    }

    // This registry is unique since I need to save the SignType, and the only way to do that is to register it. So..
    public static SignType register(String type){
        try {
            return SignTypeInvoker.register(new CherrySignType(type));
        } catch (IllegalAccessException iae){
            System.out.println("CherryRegistry threw IllegalAccessException when trying to register \"" + type + "\" sign type.");
            iae.printStackTrace();
        }
        return null;
    }

    public static void registerBurnable(Block block, int burnChance, int burnSpread){
        FlammableBlockRegistry.getDefaultInstance().add(block, burnChance, burnSpread);
    }

    public static void registerColor(Block block, int color){
        ColorProviderRegistry.BLOCK.register((state, world, pos, tintIndex) -> {
            return color;
        }, block);
    }

    public static void registerTransparentLayer(Block block){
        BlockRenderLayerMap.INSTANCE.putBlock(block, RenderLayer.getCutout());
    }

    public static void addStrippable(Block block, Block stripped){
        STRIPPABLE_BLOCKS.put(block, stripped);
    }
}
